% Daysecs.pro: Abstractions about day seconds.
% Author: Frans A. Meijs, 2003.

% This class is particularly intended for interval calculations.
% For presentation purposes it can be mapped into class TOD (time of 
% day).

mapping(['daySecs__incrBy_nat__is_daySecs',DSS1,N,DSS2]) :-
   daySecs__incrBy_nat__is_daySecs(DSS1,N,DSS2).


daySecs__incrBy_nat__is_daySecs(DSS,0,DSS).
daySecs__incrBy_nat__is_daySecs(DSS1,N,DSS2) :-
   nonvar(DSS1),
%   nonvar(N),             % Covered by nat/1 term.
   nat(N),
   maxSecondsInDay(MD),
   DSS2 is (DSS1 + N) mod MD.
daySecs__incrBy_nat__is_daySecs(DSS1,N,DSS2) :-
   var(N),
   nonvar(DSS1),
   nonvar(DSS2),
   int__lt_int(DSS1,DSS2),
   N is DSS2 - DSS1.
daySecs__incrBy_nat__is_daySecs(DSS1,N,DSS2) :-
   arityOfVars__gt_1(DSS1,N,DSS2),
   o_exceptionMsg('daySecs__incrBy_nat__is_daySecs',
                    [DSS1,N,DSS2],
                    'is queried with two unbound arguments'),
   fail.

% MAX/MIN operators etc. can be taken from Prolog/int.pro


daySecs(DSS) :-
   nat(DSS),
   maxSecondsInDay(MD),
   int__le_int(DSS,MD).

% :- consult('daysecs.prp').

