% SDV.PRO: Abstractions about sequential document volumes.
% Author: Frans A. Meijs, 1999

% All addresses and time intervals are in seconds.

docVol(DV) :- seqDocVol(DV).

elemState__e_entity(D,SDV) :-
   duration__e_seqDocVol(D,SDV).
       % This way of specification has been chosen because the duration
       % is defined to be a constant. When constant elemStates like this
       % were made identical to state variables with fixed state, everything
       % with some constant elemState would become a structural unit.

beginAddress__e_docVolMembership(BA, DVM) :-
   beginAddress__e_seqDocVolMembership(BA, DVM).

elemState__e_entity(IDG,VT) :-
   interDocGap__e_seqDocVol(IDG,VT).


interDocGap__e_seqDocVol([BA,GD],SDV) :-
   seqDocVol(SDV),
   docVol__e_docVolMembership(SDV,DVM0),
   document__e_docVolMembership(SD0,DVM0),
   beginAddress__e_docVolMembership(BA0,DVM0),
   docVol__e_docVolMembership(SDV,DVM1),
   beginAddress__e_docVolMembership(BA1,DVM1),
   BA1 > BA0,
   ~( beginAddress__isIndirectSuccessorOf_beginAddress__on_seqDocVol(
                                                        BA1,BA0,SDV) ),
                       % The begin address of the gap:
   seqDocVolRelTime__eq_address(RT0,BA0),
   duration__e_seqDoc(D0,SD0),
   RT is RT0 + D0,
   seqDocVolRelTime__eq_address(RT,BA),
                       % Now the gap duration:
   seqDocVolRelTime__eq_address(RT1,BA1),
   GD is RT1 - RT.
                     % The trailing gap on the volume:
interDocGap__e_seqDocVol([BA,GD],SDV) :-
   seqDocVol(SDV),
   docVol__e_docVolMembership(SDV,DVM0),
   document__e_docVolMembership(SD0,DVM0),
   beginAddress__e_docVolMembership(BA0,DVM0),
   duration__e_seqDocVol(VTD,SDV),
   seqDocVolRelTime__eq_address(VTD,BA1),
   ~( beginAddress__isIndirectSuccessorOf_beginAddress__on_seqDocVol(
                                                        BA1,BA0,SDV) ),
                       % The begin address of the gap:
   seqDocVolRelTime__eq_address(RT0,BA0),
   duration__e_seqDoc(D0,SD0),
   RT is RT0 + D0,
   seqDocVolRelTime__eq_address(RT,BA),
                       % Now the gap duration:
   GD is VTD - RT.
                     % An empty volume:
interDocGap__e_seqDocVol([0,GD],SDV) :-
   seqDocVol(SDV),
   ~( docVol__e_docVolMembership(SDV,_) ),
   duration__e_seqDocVol(GD,SDV).


association([BA1, BA0, SDV]) :-
   beginAddress__isIndirectSuccessorOf_beginAddress__on_seqDocVol(BA1, BA0, SDV).
   
beginAddress__isIndirectSuccessorOf_beginAddress__on_seqDocVol(BA1, BA0, SDV) :-
   nonvar(BA1), nonvar(BA0), nonvar(SDV),
   seqDocVol(SDV),
   docVol__e_docVolMembership(SDV, DVM),
   beginAddress__e_seqDocVolMembership(BA, DVM),
   BA > BA0,
   BA < BA1.

